------------------------------------------------------------------------------------------
-- Whammy
-- Created By:  Jaycyn of Hyjal
--
-- What is it: Heal, Decurse and more with a point and click
-- License   : Distribute as you wish, please give credit to the author, please send
--           :   bugs and suggestions instead of modifying code yourself.
------------------------------------------------------------------------------------------

local WhammyVers = "0.3.7";
local WhammyDate = "2/16/2007";
W_kHealthBarWidth = 70; --the width of the health bar on each unit button

-- below are functions dealing with clicking in the unit buttons to cast spells
--   note that the default action for each button is assigned when the main frame loads.
function W_UnitButton_Onload()
	--W_ShowMessage("Unit_Button_Onload");
	this:RegisterForClicks("LeftButtonUp", "RightButtonUp", "MiddleButtonUp"); -- the buttons need to see the mouse clicks
	-- this refers to the frame/button that called this event
end



--
-- Handle Unit Button Mouse actions
--
function W_UnitButton_MouseDown() --MouseDown and up just highlight the unit name when clicked. arg1 is the LeftButton, etc. unused
	local buttonName = this:GetName(); --get the name of the unit button that triggered this event
	getglobal(buttonName.."_UnitName"):SetFontObject( W_kFontNameHighlight );
end

function W_UnitButton_MouseUp() -- unhighlight the name
	local unitID = this:GetAttribute("unit"); --get the name of the unit assigned to this button
	local buttonName = this:GetName(); --get the name of the button

	if ( unitID ) then --if this is a valid unit then
		local online = UnitIsConnected( unitID ); --get the unit status assigned to this button
		local isDead = W_IsDead( unitID );
		local isAUnit = W_IsUnit( unitID ); -- see if this is a unit instead of a pet
		
		if ( isAUnit ) then -- this is a unit and not a pet - adjust the color of the text part of the frame
			myTextFrame = getglobal( buttonName.."_UnitName" );
			if ( online ) then
				if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
					myTextFrame:SetFontObject( W_kFontNamePlayer );
				else --someone else in the party or raid
					myTextFrame:SetFontObject( W_kFontNameOthers );
				end
			else --offline, show their text disabled
				myTextFrame:SetFontObject( W_kFontNameDisable );
			end
		end
	end
end
-- End Handle Unit Button Mouse actions



--
-- below are items dealing with the main Whammy Frame: registering for events, mouse clicks etc
--
function W_Frame_Onload(self)
	-- W_ShowMessage("Frame_Onload");

	-- Register the game events neccesary for our functionality
	self:RegisterEvent("ADDON_LOADED");
	--self:RegisterEvent("VARIABLES_LOADED");
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("PLAYER_AURAS_CHANGED");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("RAID_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MANA");
	self:RegisterEvent("PLAYER_LOGIN");
	self:RegisterEvent("PLAYER_REGEN_DISABLED"); --use this to know if we enter combat so we can disable the drop downs
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	--self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	--					CHAT_MSG_SPELL_AURA_GONE_SELF
	
	
--self:RegisterEvent("PET_UI_UPDATE")
--self:RegisterEvent("PLAYER_PET_CHANGED")
--self:RegisterEvent("PET_BAR_CHANGED")
--self:RegisterEvent("UNIT_PET")
	
	
	-- set up the slash commands
	SlashCmdList["WHAMMY"] = W_Slash_Command_Handler;
	SLASH_WHAMMY1 = "/whammy";
	SLASH_WHAMMY2 = "/wham";
end

-- Handle stuff relevent to the Whammy Frame
function W_Frame_MouseDown(self) --this toggles the config frame
	--W_ShowMessage("Frame_MouseDown");
	
	if arg1 == "LeftButton" then -- drag the frame
		self:StartMoving();
	elseif arg1 == "RightButton" then --need to show a frame that lets the user turn mouseovers on or off
		W_Config_Frame_Toggle();
	end
end

function W_Toggle_Config_Frame(self)
	if arg1 == "RightButton" then --need to show a frame that lets the user turn mouseovers on or off
		local f = UIParent;
		local w = f:GetWidth();
		W_ShowMessage("width is: "..w)
	end
	
end

-- Based upon our registered events, we want different things to happen when each event occurs.
function W_Frame_OnEvent(self, event, ...)
	--W_ShowMessage( event );
	
	if ( event == "ADDON_LOADED" and arg1 == "Whammy") then -- the event occurs after the saved vars are loaded
		W_ShowMessage("Version: "..WhammyVers.." "..WhammyDate.." loaded"); --" V0.21a 1/6/2007 Loaded");
		W_ShowMessage("    /Whammy to show/hide Whammy");
		this:UnregisterEvent("ADDON_LOADED");
		gWhammyIsLoaded = true;
		W_Handle_Addon_Loaded_Event(); --set up button frames, etc
	end
	
	if ( gWhammyIsLoaded == true ) then --only take action on these events if Whammy is loaded
		if ( event == "VARIABLES_LOADED" ) then --this event occurs after we load in any saved variables. Don't do anything here. functio TBD
			--W_ShowMessage("Variables Loaded Event");
		elseif ( event =="PLAYER_LOGIN") then
			if ( GetNumRaidMembers() == 0 ) then --not a raid
				W_Handle_Party_Members_Changed_Event(); --set the frame to the appropriate size for either player or party
			else -- its a raid, set up the frame
				W_Handle_Raid_Roster_Update_Event();
			end
			
			
		elseif ( event == "PLAYER_AURAS_CHANGED" ) then --watch for this player
			W_Handle_Aura_Event("player");
		elseif ( event == "PLAYER_REGEN_DISABLED" ) then --we have entered combat
			W_Handle_Combat( true );
		elseif ( event == "PLAYER_REGEN_ENABLED" ) then --we have left combat
			W_Handle_Combat( false );
			
			
		elseif ( event == "PARTY_MEMBERS_CHANGED" ) then
			if ( GetNumRaidMembers() == 0 ) then --we only care about this event if we are not in a raid since that will
				W_Handle_Party_Members_Changed_Event(); --    generate an event as well
			end
		elseif ( event == "RAID_ROSTER_UPDATE" ) then
			W_Handle_Raid_Roster_Update_Event();
			
			
		elseif ( event == "UNIT_AURA" and arg1 ~="target" and arg1 ~= "mouseover" and arg1 ~="focus" and arg1 ~="npc" and arg1 ~= "NPC" ) then 
			W_Handle_Aura_Event( arg1 ); --arg1 is the unitID of the unit whose buff changed
		elseif ( event == "UNIT_HEALTH" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then
			W_Handle_Health_Event( arg1 ); --arg1 is player, pet, partyX, partypetX, raidX, raidpetX
		elseif ( event == "UNIT_MANA" and arg1 ~= "target" and arg1 ~= "mouseover" and arg1 ~="focus"  and arg1 ~= "npc" and arg1 ~= "NPC" ) then
			W_Handle_Unit_Mana_Event(); --update the total mana for player, party or raid, this function is in WhammySummary.lua
		end
	end
	
	
--	if ( event == "PET_UI_UPDATE" or event == "PLAYER_PET_CHANGED" or event == "PET_BAR_CHANGED"or event == "UNIT_PET") then
--		W_ShowMessage( event.."  "..arg1 );
--	end
end

--------
--------  EVENT HANDLERS --------
--------

-- Handle changes to the party or raid
function W_Handle_Addon_Loaded_Event()
	if ( W_gIsVisible == nil ) then --if this var has not been initted, then init it
		W_gIsVisible = true; --determines is the Whammy frame is visible or not
	end
	
	if ( W_gSummaryIsVisible == nil ) then
		W_gSummaryIsVisible = true;
	end

	if (W_SavedSpells == nil) then --if no saved spells were loaded
		W_Set_Saved_Spells(); --will set spell table defaults
	end
	
	if ( W_gSavedMacros == nil ) then
		W_gSavedMacros = {};
	end
	
	W_Add_Drop_Down_Options();
	
	if ( W_gHealthLevel == nil ) then
		W_gHealthLevel = "Off"; --off by default
	end
	
	W_Create_Unit_Buttons(); --create all of the unit buttons and also creates all of the potential pet buttons
	
	if ( W_gIsVisible ) then --if the last time the player played, it was visible, keep it visible
		local frame = WhammyFrame;
		if ( frame ) then
			frame:Show();
		end
	end
	
	-- show the summary frame or not
	WhammyConfigFrame_Show_Summary_CheckButton:SetChecked( W_gSummaryIsVisible );
	W_Show_Summary_CheckButton_Click( W_gSummaryIsVisible );
	
end

-- called after the spell tables have either been loaded or defaulted, to add any addtional button options: Trade, Whisper etc
function W_Add_Drop_Down_Options()
	local charType = UnitClass("player");
	local tableName = ("W_"..charType.."SpellTable"); --figure out which table to use based on char type
	local t = getglobal(tableName); -- a reference to the table
	table.insert(t, "Trade");
	table.insert(t, "Whisper");
	table.insert(t, W_kMenuRunMacro);
end



-- called when the party members change (add, leave) - will only be called for solo and party events
function W_Handle_Party_Members_Changed_Event()
	--W_ShowMessage("Party members changed event");
	local i, numberOfPartyMembers;
	local unitID, petID, buttonName;
	local numberOfPartyMembers = GetNumPartyMembers(); -- is there anyone else in the party? excluding the player ( buttons2-5)
	
	W_ResetAllButtons();
	
	W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player
	local localizedClass, englishClass = UnitClass( "player" ); --what class of player so we know which table to get the defaults from
	if ( englishClass == "HUNTER" ) then
		W_ShowMessage("  its a hunter");
		W_Set_Unit_Button_Name( "W_Pet_Button1", "pet" ); --set the button to this pet and adds the spells
	end
					
	if ( numberOfPartyMembers == 0 ) then -- the player is by themself, reset the other buttons and frame size
		W_Set_Whammy_Frame_Size( W_kPlayer );
	else -- player is in a party, populate the other player buttons and set the frame size
		for i = 1,4 do --1 through 4 are the other party members i.e. party1 is the first other party member etc
			unitID = "party"..i;
			if UnitExists( unitID ) then --if this is a valid unit then
				buttonName = "W_Unit_Button"..(i+1); --set the button to this unit
				W_Set_Unit_Button_Name( buttonName, unitID );
								
				local localizedClass, englishClass = UnitClass( unitID ); --what class of player so we know which table to get the defaults from
				if ( englishClass == "HUNTER" ) then
					petID = "partypet"..i
					buttonName = "W_Pet_Button"..(i+1);
					W_Set_Unit_Button_Name( buttonName, petID ); --set the button to this pet and adds the spells
				end
			end
		end
		W_Set_Whammy_Frame_Size( W_kParty );
	end
end

--called when the raid roster changes or upon player login if they are in a raid
function W_Handle_Raid_Roster_Update_Event()
	--W_ShowMessage("Raid Roster Update");
	local name, rank, subgroup, level, class, fileName, zone, online, isDead;
	local unitCount, pos, id;
	local unitButtonName, unitID, petButtonName, petID;
	local subGroupUnitCount = {0,0,0,0,0,0,0,0}; --keep track of how many raid members are in each raid group. Starts at 0 (5 per group)
	W_gUnitButtonTable = {}; --clear out the global lookup table that links player unitID'S to the unit buttons they are in,
	W_gPetButtonTable = {};
	
	--W_ShowMessage("Raid Roster Update")
	
	local numRaidMembers = GetNumRaidMembers();
	
	W_ResetAllButtons();

	if ( numRaidMembers == 0 ) then -- if there are no raid members, reset size etc to just the player
		W_Set_Unit_Button_Name("W_Unit_Button1","player"); --set the unit button 1 to this player
		local localizedClass, englishClass = UnitClass( "player" ); --what class of player so we know which table to get the defaults from
		if ( englishClass == "HUNTER" ) then
			W_Set_Unit_Button_Name( "W_Pet_Button1", "pet" ); --set the button to this pet and adds the spells
		end
		W_Set_Whammy_Frame_Size( W_kPlayer ); --set the frame size to just player
	else
		W_Set_Whammy_Frame_Size( W_kRaid );
		for id = 1,numRaidMembers do --walk through all the raid members
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id);	--get their name and subgroup
			subGroupUnitCount[ subgroup ] = subGroupUnitCount[ subgroup ] + 1; --keep track of how many units are in each subgroup
	
			unitCount = subGroupUnitCount[ subgroup ]; --get the next unit position based on the unit count of the subgroup
			
			pos = ( ( (subgroup-1) * 5) + unitCount ) -- first subgroup, first position will be 1, 8th subgroup, first position will be 36
			
			unitButtonName = "W_Unit_Button"..pos
			unitID = "raid"..id
			W_Set_Unit_Button_Name( unitButtonName, unitID );
			W_gUnitButtonTable[ unitID ] = unitButtonName; --add the buttons that are occupied to the table so we can look them up later
			
			local localizedClass, englishClass = UnitClass( unitID ); --what class of player is this: used to populate Hunter pet buttons
			if ( englishClass == "HUNTER" ) then
				petButtonName = "W_Pet_Button"..pos;
				petID = "raidpet"..id
				W_Set_Unit_Button_Name( petButtonName, petID ); --set the button to this pet and adds the spells
				W_gPetButtonTable[ petID ] = petButtonName;
			end
		end
	end
end

-- handle unit health events for player or someone in the players party (1-4)
function W_Handle_Health_Event( unitID )
	--W_ShowMessage("Handle Party Unit Health Event for "..unitID):
	local myButtonName;
	local online, isDead;
	local partyMemberNum;
	local myTextFrame;
	local healthBarAmount;
	local isAUnit = W_IsUnit( unitID ); -- is this a unit as opposed to a pet
	
	myButtonName = W_Get_Button_Name_From_UnitID( unitID ); --returns W_Unit_ButtonX or W_Pet_ButtonX
	
	if ( myButtonName ~= nil ) then
		online = UnitIsConnected( unitID );
		isDead = W_IsDead( unitID );
		
		if ( isAUnit ) then -- this is a unit and not a pet - adjust the color of the text part of the frame
			myTextFrame = getglobal( myButtonName.."_UnitName" );
			if ( online ) then
				if ( UnitIsUnit( unitID, "player") ) then --if they are online, color the player one way, and the rest of the units another
					myTextFrame:SetFontObject( W_kFontNamePlayer );
				else
					myTextFrame:SetFontObject( W_kFontNameOthers );
				end
			else --offline, show their text disabled
				myTextFrame:SetFontObject( W_kFontNameDisable );
			end
		end
		
		if ( isDead ) then --dead so show no health and adjust button texture
			W_Set_Unit_Button_Texture( myButtonName, W_kUnitDead ); --unit or pet is dead, grey out the unit button texture
			W_Set_Unit_Health_Bar_Value( myButtonName, 1); --set the healthbar to 1 (0 doesnt work)
		else -- unit or pet is alive, update health bar
			local maxHealth = UnitHealthMax(unitID);
			local currentHealth = UnitHealth(unitID);
			local healthFraction = currentHealth / maxHealth;
			if ( isAUnit ) then --if its a unit
				healthBarAmount = W_kHealthBarWidth * healthFraction;
			else --its a pet, use the pet healthbar width
				healthBarAmount = W_kPetHealthBarWidth * healthFraction;
			end

			W_Set_Unit_Button_Texture( myButtonName, W_kUnitAlive);
			W_Set_Unit_Health_Bar_Value( myButtonName, healthBarAmount );
			
			if W_gHealthLevel ~= "Off" then --if the user wants to be notified for low health, let them know
				--if the unit has low health, make the texture of the unit button red to get attention
				local healthTargetPercent = tonumber(W_gHealthLevel)/100; --health level is the % health selected in the drop down
				if ( healthFraction < healthTargetPercent ) then -- w_gHealthLevel is a saved var
					W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthAlert )
				else --good health, set to default color
					W_Set_Unit_Button_Texture( myButtonName, W_kUnitHealthOK )
				end
			end
			
		end
	end
end

function W_IsAPet( unitID )
	return false;
end

function W_IsDead( unitID )
	local isDead = (   UnitIsDeadOrGhost( unitID ) or ( UnitHealth( unitID ) <= 0 ) or UnitIsCorpse( unitID )   )
	
	return isDead
end


function W_Set_Unit_Button_Texture( buttonName, whichTexture ) --set the texture of the entire unit button, either W_Unit_Button1 or W_Pet_Button1
	--W_ShowMessage("Set Unit Button Texture for unit "..unitID.." texture is "..whichTexture );
	local r, g, b, a;
	local isAUnit = W_Get_Unit_Name_From_Button( buttonName ); --if isAUnit is nil, then it's a pet button because pet buttons dont have text in them

	if ( buttonName ~= nil) then
		if ( isAUnit ) then --if this is a unit do unit colors
			if ( whichTexture == W_kUnitAlive ) then
				r = 1; g = 1; b = 1; a = .2;	
				
			elseif ( whichTexture == W_kUnitDead ) then
				r = .1; g = .1; b = .1; a = .8;
				
			elseif ( whichTexture == W_kUnitHealthAlert ) then
				r = 1; g = 0; b = 0; a = .5;
				
			elseif ( whichTexture == W_kUnitHealthOK ) then
				r = 1; g = 1; b = 1; a = .2;
				
			elseif ( whichTexture == W_kUnitDisconnected ) then
				r = 0; g = 0; b = 0; a = 0;	
				
			else
				r = 1; g = 1; b = 1; a = .2;
			end
		else --must be a pet, so do pet colors
			--W_ShowMessage("   setting pet texture for button "..buttonName.." texture is "..whichTexture);
			if ( whichTexture == W_kUnitAlive ) then
				r = .2; g = .2; b = .2; a = 1;	

			elseif ( whichTexture == W_kUnitDead ) then
				r = .1; g = .1; b = .1; a = 1;
				
			elseif ( whichTexture == W_kUnitHealthAlert ) then
				r = 1; g = 0; b = 0; a = 1;
				
			elseif ( whichTexture == W_kUnitHealthOK ) then
				r = .2; g = .2; b = .2; a = 1;
				
			elseif ( whichTexture == W_kUnitDisconnected ) then
				r = 0; g = 0; b = 0; a = 0;	
				
			else
				r = .2; g = .2; b = .2; a = 1;
			end
		end
		
		local buttonTextureName = buttonName.."_Texture";
		--W_ShowMessage("       buttonTextureName="..buttonTextureName)
		getglobal( buttonTextureName ):SetTexture( r, g, b, a );
	end
end

function W_Set_Unit_Health_Bar_Value( buttonName, healthBarValue ) --set the level of the health bar on the unit button
	local buttonHealthBarName = ( buttonName.."_HealthBar" ); --so we have W_Unit_Button1_HealthBar
	getglobal( buttonHealthBarName ):SetWidth( healthBarValue ); --adjust the width of the healthbar
	
	W_Update_Total_Health(); -- update the total health bar for the player, party or raid
end

function W_Set_Unit_Button_Curse_Texture( buttonName, isDebuffed) --set the texture of the curse area on the unit button
	local buttonCurseTextureName = ( buttonName.."_Curse_Texture" );

	if ( isDebuffed ) then
		getglobal( buttonCurseTextureName ):SetTexture( .97, 1, .38, 1 ); --yellow
	else
		getglobal( buttonCurseTextureName ):SetTexture( nil );
	end
end

-- called when the player/pet, a party/partypet or a raid member/raid pet  buffs/debuffs change.
function W_Handle_Aura_Event( unitID )
	--W_ShowMessage( "Unit Aura Event for unitid "..unitID);
	local debuffName, debuffRank, debuffTexture, debuffApplications, debuffType;
	local i = 1;
	local showDispellable = 1; -- 1 = only return debuffs that are dispellable, 0 = all buffs
	local foundOne, unitButtonName;

	unitButtonName = W_Get_Button_Name_From_UnitID( unitID ); --returns W_Unit_ButtonX or W_Pet_ButtonX
	
	if ( unitButtonName ~= nil) then
		while ( UnitDebuff( unitID, i) ) do -- while the unit has debuffs, walk through them as see if there are any we can fix
			debuffName, debuffRank, debuffTexture, debuffApplications, debuffType = UnitDebuff( unitID, i, showDispellable );
			if (debuffTexture ~= nil) then
				foundOne = true;
				break; -- found one so exit the loop
			end
			i=i+1;
		end
		if (foundOne) then -- we found at least 1 debuff we can dispell - highlight the units button
			W_Set_Unit_Button_Curse_Texture( unitButtonName, true);
		else -- nope, no debuffs, set the unit button back to normal
			W_Set_Unit_Button_Curse_Texture( unitButtonName, false);
		end
	end
end

--determine the button name from the unitID. Pass in player, pet, party1-4, partypet 1-4, raid 1-40 or raidpet1-40
--   return the button name that refers to this unit or pet either W_Unit_ButtonX or W_Pet_ButtonX
function W_Get_Button_Name_From_UnitID( unitID )
	local b;
	local id;
	
	--W_ShowMessage( "Getting Button name for unit "..unitID );
	
	if ( unitID == "player" ) then
		b = "W_Unit_Button1"
	elseif ( unitID == "pet" ) then
		b = "W_Pet_Button1"
	else --we are either in a party or a raid, and could be a unit or a pet
		if ( W_IsRaid() ) then
			--W_ShowMessage(" W_Get_Button_Name_From_UnitID "..unitID);
			b = W_Get_Raid_Unit_Button_From_UnitID( unitID ); --look up which button this raid unit is in via the global table
		else --its a party
			if ( W_IsUnit( unitID ) ) then --its a player unit
				id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
				b = "W_Unit_Button"..(id+1);
			else --its a pet
				id, numberFound = gsub( unitID, "partypet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
				b = "W_Pet_Button"..(id+1)
			end
		end
	end

	return b
end

-- called when we enter or leave combat. Enable or disable the drop downs depending on combat status
function W_Handle_Combat( isInCombat )
	if ( isInCombat ) then
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			local dropDownName = "W_DropDown"..i
			UIDropDownMenu_DisableDropDown( getglobal( dropDownName ) );
		end
	else
		for i = 1,W_kMaxDropDownButtons do --walk through the drop downs and enable or disable them depending on if we are in combat
			local dropDownName = "W_DropDown"..i
			UIDropDownMenu_EnableDropDown( getglobal( dropDownName ) );
		end
	end
end
--
--------  END EVENT HANDLERS --------
--


--
-- functions called to set up the unit buttons to respond to mouse clicks
--
-- called from the handle raid roster update and party change events to assign units to unit buttons
function W_Set_Unit_Button_Name( buttonName, unitID)
	--W_ShowMessage( "Setting Unit Button "..buttonName.." to "..unitID);
	
	getglobal(buttonName):SetAttribute("unit", unitID); --set the assignment of the button to the passed in unit  (need to do this first)
	
	if ( W_IsUnit( unitID ) ) then --if it is a unit instead of a pet, put the units name in the button
		local b = getglobal( buttonName.."_UnitName" );
		local unitName = UnitName(unitID); -- unit ID will be "player" or "party1" or "raid1", and this will give us the actual name
		b:SetText(unitName); --set the text of the passed in button to the unit name
		
		if ( UnitIsUnit( unitID, "player" ) ) then -- if this button is the player button, make it stand out so it is more easily picked out
			b:SetFontObject( W_kFontNamePlayer );
		else
			if ( UnitIsConnected( unitID ) ) then --if the unit is connected, standard font color
				b:SetFontObject( W_kFontNameOthers );
			else --if the unit disconnects then change the font color to disconnected color
				b:SetFontObject( W_kFontNameDisable );
			end
		end
	else --it was a pet, make sure the button is visible
		getglobal(buttonName):Show()
	end
	
	W_Set_Unit_Button_Click( buttonName );
end

--walk through and update all of the unit buttons to reflect a change in spell or unit etc
function W_Update_Unit_Buttons() 
	for i = 1,W_kMaxUnitButtons do --now that we changed a spell, walk through and update all of the unit buttons to reflect the change
		W_Set_Unit_Button_Click( "W_Unit_Button"..i );
		W_Set_Unit_Button_Click( "W_Pet_Button"..i);
	end
end

--called from handle raid roster update, dropdown selection change events and default button click.
--      Assign mouse clicks to the passed in unit button
function W_Set_Unit_Button_Click( buttonName )
	--W_ShowMessage("setting ".. unitButtonName);
	W_Set_Click("", "1", buttonName, "W_DropDown1" ); --Left Click is DropDown1
	W_Set_Click("", "2", buttonName, "W_DropDown3" ); -- Right Click is DropDown3
	W_Set_Click("", "3", buttonName, "W_DropDown2" ); --Middle Button Click is DropDown2
	W_Set_Click("ctrl", "1", buttonName, "W_DropDown4" ); --Ctrl Left
	W_Set_Click("ctrl", "2", buttonName, "W_DropDown6" ); --Ctrl Right
	W_Set_Click("ctrl", "3", buttonName, "W_DropDown5" ); --Ctrl Middle
	W_Set_Click("shift", "1", buttonName, "W_DropDown7" ); --Shift Left
	W_Set_Click("shift", "2", buttonName, "W_DropDown9" ); --Shift Right
	W_Set_Click("shift", "3", buttonName, "W_DropDown8" ); --Shift Middle
	W_Set_Click("alt", "1", buttonName, "W_DropDown10" ); --Alt Left
	W_Set_Click("alt", "2", buttonName, "W_DropDown12" ); --Alt Right
	W_Set_Click("alt", "3", buttonName, "W_DropDown11" ); --Alt Middle
end

-- Called from W_Set_Unit_Button_Click to set what the mouse click does in reponse to the players frameButton
function W_Set_Click(keyPress, mouseButton, buttonName, ddMenuName )
	--keyPress will be filled if there is a keyboard button pressed along with the mouse click
	--mouseButton is: 1=Left, 2=Right, 3=Middle, 4 and 5
	--buttonName is the frame button on the screen we are assigning the clicks to
	--ddMenuName is the dropdown menu name this spell is saved for
	
	local myUnitID, myUnitName, myButton, menuItem, macroName;
	
	myButton = getglobal( buttonName ); --get a pointer to the unitButton that we are configuring
	
	if ( myButton:GetAttribute("unit") ~= nil ) then -- the unit attrib would be nil if it is does not have a unit assigned to it
	
		if ( keyPress == "ctrl" or keyPress == "shift" or keyPress == "alt") then --if there is a keypress modifier then we need to add 
			keyPress = ( keyPress.."-" ); --                                       the dash so the attribute will be properly formatted
		end
		
		menuItem, macroName = W_Get_Saved_MenuItem( ddMenuName );--get the item from the saved spell table: spell, Whisper, Trade or W_kMenuRunMacro

		if ( menuItem == "Trade" ) then --this button will be set to trade
			myUnitID = myButton:GetAttribute("unit");
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Trade");
			myButton.W_Perform_Trade = function() InitiateTrade(myUnitID); W_ShowMessage("Opening trade window"); end;
			
		elseif ( menuItem == "Whisper") then
			myUnitID = myButton:GetAttribute("unit");
			myUnitName = UnitName( myUnitID );
			local chatType = "WHISPER";
			local channel = GetChannelName( chatType );
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "W_Perform_Chat_Message");
			myButton.W_Perform_Chat_Message = function() ChatFrame_SendTell(myUnitName); W_ShowMessage("Whisper"); end;
			
		elseif ( menuItem == W_kMenuRunMacro) then
			--W_ShowMessage( "W_SetClick detected a macro"..macroName );
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "macro");
			myButton:SetAttribute( (keyPress.."macro"..mouseButton), macroName); --which spell is actually a macro here.

		else --we are setting the button to a spell cast or action
			myButton:SetAttribute( (keyPress.."type"..mouseButton), "spell");
			myButton:SetAttribute( (keyPress.."spell"..mouseButton), menuItem);
		end
	end
end

function W_Open_Trade_Macro()
	InitiateTrade("target");
	W_ShowMessage("opening trade window");
end

function W_ResetAllButtons()
	for i = 1,W_kMaxUnitButtons do --clear out the raid buttons
		local unitButtonName = "W_Unit_Button"..i
		local petButtonName = "W_Pet_Button"..i
		local ubTextFrame = getglobal( unitButtonName.."_UnitName" );
		local ub = getglobal( unitButtonName );
		local pb = getglobal( petButtonName);
		
		-- set the text of the button to -
		ubTextFrame:SetText("-");
		ubTextFrame:SetFontObject( W_kFontNameOthers );
		
		-- set the action of the button to nil
		ub:SetAttribute("type", "stop"); --set the function of the button to nothing
		ub:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
		
		-- set the color of the button to standard
		W_Set_Unit_Button_Texture( unitButtonName, W_kUnitHealthOK );
		
		-- set the curse of the button to off
		W_Set_Unit_Button_Curse_Texture( unitButtonName, false);
		
		-- set the health bar to full health
		W_Set_Unit_Health_Bar_Value( unitButtonName, W_kHealthBarWidth);
		
		
		pb:SetAttribute("type", "stop"); --set the function of the button to nothing
		pb:SetAttribute("unit", nil); --set the unit attrib (the unit name) to nothing
		pb:Hide();
		
		-- set the color of the button to standard
		W_Set_Unit_Button_Texture( petButtonName, W_kUnitHealthOK );
		
		-- set the curse of the button to off
		W_Set_Unit_Button_Curse_Texture( petButtonName, false);
		
		-- set the health bar to full health
		W_Set_Unit_Health_Bar_Value( petButtonName, W_kPetHealthBarWidth);
	end
end

--
-- W_DropDownMenu stuff: the onload function defines which function init's the button (which is automatically called)
--
-- load up the action buttons with default spells if there were no saved spells
function W_Set_Saved_Spells()
	W_ShowMessage("Setting spells to default");
	if ( W_SavedSpells == nil ) then
		W_SavedSpells = {}; --create the table
	end
	
	local class = UnitClass("player"); --what class of player so we know which table to get the defaults from
	local defaultTableName = "W_"..class.."DefaultSpellTable"; -- the default spell table for this class ie W_PriestDefaultSpellTable

	for i = 1,W_kMaxDropDownButtons do --for each drop down button, get what it's default should be from the default tables for this char type
		local dropDownButtonName = "W_DropDown"..i;
		W_SavedSpells[ dropDownButtonName ] = getglobal(defaultTableName)[i]; -- this is W_DropDown1="Greater Heal" and W_DropDown2="Renew" etc
		--W_ShowMessage( dropDownButtonName.." = "..getglobal(defaultTableName)[i] );
	end
	
	--W_ShowMessage( "Saved Spell Count = "..table.getn(W_SavedSpells) );
end

--	use for troubleshooting
function W_Show_Spells() -- this function is for troubleshooting
	for k, v in pairs(  W_SavedSpells ) do --iterate over the key, values pairs from the spell table appropriate for this char
		local value = W_SavedSpells[k];
	end
end


--
-- MISC functions from here down
--

-- Display a message in the chat frame
function W_ShowMessage(aMessage)
	local r,g,b,a
	r = 0;
	g = 1;
	b = 0;
	a = .5;
	
	if ( aMessage ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: " ..aMessage, r, g, b, a);
	else
		DEFAULT_CHAT_FRAME:AddMessage("Whammy: nil message");
	end
end

-- handle clicking the set defaults button - need to add an Are You Sure dialog
function W_Default_Button_Click() --clicking the button sets the spells back to their defaults
	--W_ShowMessage("Button click");
	
	W_Set_Saved_Spells(); --set the saved spell table to the default spells for this type of char (priest, mage etc)
	
	for i = 1,W_kMaxDropDownButtons do --now walk through the buttons and set their text
		local dropDownName = "W_DropDown"..i --each drop down
		local ddFrame = getglobal( dropDownName ); -- and the drop down frame
		local spellName = W_SavedSpells[dropDownName]; -- and the spells that we want to default the drop down to
		--W_ShowMessage( dropDownName.." = "..spellName );
		UIDropDownMenu_SetSelectedName( ddFrame, spellName); --set the selected item in the list
		UIDropDownMenu_SetText( spellName, ddFrame ); --also put the name of the spell in the text area
	end
	
	W_Update_Unit_Buttons();
	
	-- set the health value drop down to default
	UIDropDownMenu_SetSelectedName( W_Health_DropDown, "Off" ); --set the dropdown selection
	W_gHealthLevel = "Off"
end

function W_Help_Test_Button_Click()
	W_ShowMessage("help test button");
end

function W_Test_Button_Click()
	W_ShowMessage("test button");
end



function W_Info_Button_Click() -- show/hide the info frame when the user clicks the ? button
	local frame = WhammyInfoFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
end

function W_Info_Onload(self)
	--W_ShowMessage("info onload");
	local infoText
	WhammyInfoFrame:SetWidth( 400 ); --adjust the size of the frame
	WhammyInfoFrame:SetHeight( 365 );
	
	WhammyInfoFrame_Heading:SetText("Whammy "..WhammyVers);
	
	WhammyInfoFrame_Text:SetPoint("TOPLEFT", "WhammyInfoFrame", 10, -35);
	WhammyInfoFrame_Text:SetJustifyH("LEFT");
	
			   infoText ="Whammy enables you to quickly heal, decurse, and perform|n";
	infoText = infoText.."   other functions to yourself, party or raid members through|n";
	infoText = infoText.."   a simple point and click interface. There is also a group|n";
	infoText = infoText.."   summary window that shows overall group health/mana|n";
	infoText = infoText.."|n";
	infoText = infoText.."|n";
	infoText = infoText.."/whammy             - shows/hides the Whammy window|n";
	infoText = infoText.."|n";
	infoText = infoText.."To drag Whammy - Left Click and Drag title bar|n";
	infoText = infoText.."|n";
	infoText = infoText.."Configure Clicks   - Right Click in title bar|n";
	infoText = infoText.."|n";
	infoText = infoText.."Use Health Alert to set the percentage health to notify you|n";
	infoText = infoText.."   when a unit needs healing. The Unit Button will turn Red|n";
	infoText = infoText.."   when their health drops below this point.|n";
	infoText = infoText.."|n"
	infoText = infoText.."Each Unit Button has:|n"
	infoText = infoText.."    a health bar at the bottom (in green)|n";
	infoText = infoText.."    a curse/ailment bar at the top (yellow when cursed)|n";
	infoText = infoText.."|n";
	infoText = infoText.."Your character text always appears in green to easily|n";
	infoText = infoText.."   distinguish it from other characters.|n";
	infoText = infoText.."|n";
	infoText = infoText.."Hunter pet buttons are located inside the unit button|n";
	infoText = infoText.."|n";
	infoText = infoText.."Whammy Summary  - Show/Hide the via the config menu|n";
	infoText = infoText.."                                - Left click to drag|n";
	infoText = infoText.."|n";

	WhammyInfoFrame_Text:SetText(infoText);
end


-- called to adjust the whammy frame size depending on single player, party or raid size
function W_Set_Whammy_Frame_Size( aGroupType )
	local newWidth, newHeight, numToShow;
	
	--aGroupType = W_kRaid;

	if ( aGroupType == W_kPlayer ) then
		newWidth = 99; newHeight = 60; numToShow = 1; --60 and 1 is default size for single player
	elseif ( aGroupType == W_kParty ) then
		newWidth = 99; newHeight = 143; numToShow = 5;
	elseif ( aGroupType == W_kRaid ) then
		newWidth = 318; newHeight = 254; numToShow = 40;
	else
		newWidth = 100; newHeight = 80; numToShow = 1;
	end
	
	for i = 1,numToShow do --show the unit buttons that need to be seen
		local buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Show();
	end
	
	for i = numToShow+1,40 do --hide the unused unit buttons
		local buttonName = "W_Unit_Button"..i
		getglobal( buttonName ):Hide();
		buttonName = "W_Pet_Button"..i
		getglobal( buttonName):Hide()
	end
	
	WhammyFrame:SetWidth( newWidth ); --adjust the size of the frame
	WhammyFrame:SetHeight( newHeight );
end

-- create all of the potential unit buttons. They start off hidden except for #1, which by default is the player
function W_Create_Unit_Buttons()
	for i = 1,40 do
		W_Create_Button(i);
	end
end

function W_Create_Button( buttonNum )
	local row, col;
	local buttonFrameName = "W_Unit_Button"..buttonNum;
	local buttonWidth = 70;
	local buttonHeight = 18;
	local leftMargin = 15;
	local gutter = 3;
	local topMargin = 28;
	local additionalTopMargin = ( buttonHeight * 4 ) + ( gutter * 3 ) + topMargin;
	
	if ( buttonNum < 6 ) then
		col = 0; row = buttonNum - 1;
	elseif ( buttonNum > 5 and buttonNum < 11 ) then
		col = 1; row = buttonNum - 5 - 1;
	elseif (buttonNum > 10 and buttonNum < 16) then
		col = 2; row = buttonNum - 10 - 1;
	elseif (buttonNum > 15 and buttonNum < 21) then
		col = 3; row = buttonNum - 15 - 1;
		
	elseif (buttonNum > 20 and buttonNum < 26) then
		col = 0; row = buttonNum - 20 - 1;
	elseif (buttonNum > 25 and buttonNum < 31) then
		col = 1; row = buttonNum - 25 - 1;
	elseif (buttonNum > 30 and buttonNum < 36) then
		col = 2; row = buttonNum - 30 - 1;
	elseif (buttonNum > 35 and buttonNum < 41) then
		col = 3; row = buttonNum - 35 - 1;
	end
	
	local x = leftMargin + (buttonWidth * col) + (gutter * col);
	local y = topMargin + (buttonHeight * row) + (gutter * row);
	
	if ( buttonNum > 20 ) then --if this is members 20-40, move them down below the first 20
		y = y + additionalTopMargin;
	end
	
	--create the frame: pass the  type, name, child of, template
	local f = CreateFrame("Button", buttonFrameName, WhammyFrame, "WhammyUnitButtonTemplate");

	f:SetPoint("TOPLEFT", "WhammyFrame", "TOPLEFT", x, -y);
	f:SetID(buttonNum);
	f:Hide(); --start off hidden
	
	W_Create_PetButton( buttonNum, buttonFrameName ); -- create a potential pet button beside each unit button
end
-- End Unit button creation


-- toggle the config frame shown and hidden
function W_Config_Frame_Toggle()
	local configFrame = WhammyConfigFrame;
	local wFrame = WhammyFrame;
	local parentFrame = UIParent; --need to make sure we open the config frame in the work area
	
	if ( configFrame ) then
		if ( configFrame:IsVisible() ) then --hide the frame if it's visible
			configFrame:Hide();
		else --show the frame if it's hidden

			local parentFrameWidth = parentFrame:GetWidth(); -- get the width of the main window
			local configFrameWidth = configFrame:GetWidth();
			local wFrameLeft = floor( wFrame:GetLeft() + 0.5); --get left and make it in integer
			local wFrameTop = floor( wFrame:GetTop() + 0.5);
			local wFrameWidth = floor( wFrame:GetWidth() + 0.5);
			
			local totalWidth = wFrameLeft + wFrameWidth + configFrameWidth
			
			configFrame:ClearAllPoints(); --releases the frame so it can be repositioned
							
			if ( totalWidth > parentFrameWidth ) then --if the config window is off the right side of the screen, open on the left of Whammy
				--obj:SetPoint(point, frame, relativePoint , x, y);
				configFrame:SetPoint("TOPRIGHT", wFrame, "TOPLEFT", -5, -5)
			else -- it was not off the right side of the screen, so open it on the right
				configFrame:SetPoint("TOPLEFT", wFrame, "TOPRIGHT", 5, -5)
			end
			
			configFrame:Show();
			
		end
	end
end

function W_Show_Summary_CheckButton_Click( isChecked )
	local frame = WhammySummaryFrame
	if ( frame ) then
		if ( isChecked ) then
			frame:Show();
			W_gSummaryIsVisible = true;
		else
			frame:Hide();
			W_gSummaryIsVisible = false;
		end
	end
end

-- handle our one and only slash command
function W_Slash_Command_Handler(msg)
	--W_ShowMessage("slash handler");
	local frame = WhammyFrame;
	if ( frame ) then
		if ( frame:IsVisible() ) then
			frame:Hide();
			W_gIsVisible = false;
			if ( WhammyConfigFrame:IsVisible() ) then
				WhammyConfigFrame:Hide();
			end
		else
			frame:Show();
			W_gIsVisible = true;
		end
	end
end

--get the unit name from the button passed in
function W_Get_Unit_Name_From_Button( aButton )
	--W_ShowMessage("Get Unit Name From Button "..aButton);
	local unitButtonName = aButton.."_UnitName";
	local unitName = getglobal(unitButtonName):GetText();
	
	return unitName;
end

-- get an id from the unitID - pass in either party1-party4 or partypet1-partypet4 or raid1-raid40 or raidpet1-raidpet40
function W_GetID( unitID ) --unit type will be either unit or pet
	local id, numberFound;
	
	if ( W_IsUnit( unitID ) ) then -- make sure this is unit, not a pet
		if ( GetNumRaidMembers() == 0 ) then -- party
			id, numberFound = gsub( unitID, "party([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		else
			id, numberFound = gsub( unitID, "raid([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		end
	else -- it was a pet, get it's id
		if ( GetNumRaidMembers() == 0 ) then
			id, numberFound = gsub( unitID, "partypet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		else
			id, numberFound = gsub( unitID, "raidpet([1-4]+)", "%1"); --will return 1-4 as the id's which correspond to buttons 2-5
		end
	end
	
	if ( numberFound > 0 ) then
		return id;
	else
		return nil;
	end
end

function W_IsUnit( unitID )
	if ( string.find( unitID, "pet") == nil ) then --look for pet in the unitID, which could be pet, partypet or raidpet
		return true
	else
		return false
	end
end

function W_IsPet( unitID )
	if ( string.find( unitID, "partypet") ~= nil ) then
		return true
	else
		return false
	end
end

function W_IsRaid()
	local numRaidMembers = GetNumRaidMembers();
	
	if ( numRaidMembers == 0 ) then
		return false;
	else
		return true;
	end
end

-- for a raid: return the unit button the unit is in, or the unit button the raid p
function W_Get_Raid_Unit_Button_From_UnitID( unitID ) --this is passed raid1-40, or raidpet1-40
	if ( W_IsUnit( unitID ) ) then
		local buttonName = W_gUnitButtonTable[ unitID ]; --this table is filled in whenever there is a raid roster update
	else
		local buttonName = W_gPetButtonTable[ unitID ];
	end
	
	return buttonName
end

--retrieve a saved spell for the passed in drop down
function W_Get_Saved_MenuItem( ddName )
	local menuItem, macroName;
	
	menuItem = W_SavedSpells[ ddName ]; --this is the actual name of a spell, Whisper, Trade or W_kMenuRunMacro
	
	if (menuItem == W_kMenuRunMacro) then --it was a run macro item, so retrieve the macro from the saved macros table for this dropdown
		macroName = W_gSavedMacros[ ddName ];
	end
	
	return menuItem, macroName --macro name will be nil if there is no menu item
end