--DropDownHandler

function W_DropDownHandler_OnShow()
	local dd = getglobal("DropDownTest");
	UIDropDownMenu_Initialize( this, W_DropDownHandler_Initialize);
	local text = W_MenuItems[1] --init to the first item in the list
	UIDropDownMenu_SetText(text, dd);
	UIDropDownMenu_SetWidth( 130, dd );
end

function W_DropDownHandler_Initialize()
	local dd = getglobal("DropDownTest");
	local selectedText = UIDropDownMenu_GetText( dd ); --get the option that is selected in the dropdown
	
	if ( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then -- are we initting main menus
		local index, menuItem;
		local t = W_MenuItems; --figure out which table to use based on char type
		local func = function() W_DropDownHandler_OnClick() end; --the function to be called when a dropdown item is selected
		
		for index, menuItem in pairs( t ) do --iterate over the key, values pairs from the spell table for this char
			info = UIDropDownMenu_CreateInfo();

			if ( type(menuItem) == "table" ) then --if this menu item is a table,then add the sub menu arrow
				info.text = menuItem[1]; --the first item in the sub table is the name we want displayed in the mail menu
				info.value = menuItem; --keep track of the subtable that the sub values are in so we can show them when the user selects it
				info.func = nil; --since this is a sub table, there is not function
				info.hasArrow = 1; --it does have an arrow since it is a sub menu
			else --it was not a table, so add it as a regular menu item
				info.text =  menuItem; --the text to show in the menu
				info.value = menuItem; --the value ( the index ) of the item
				info.func = func; --the function to call when the item is selected
				info.checked = (menuItem == selectedText) --if this is the item that is selected, then check it
			end
			UIDropDownMenu_AddButton( info, 1 );
		end
	elseif ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then -- are we initting sub menus
		local subMenuTable = UIDROPDOWNMENU_MENU_VALUE; --the value for this is set above in info.value=menuitem, where menuitem is a table
		local subMenuName = subMenuTable[1]; --the first item in the sub menu table is the name
		local func = function() W_DropDownHandler_OnClick() end; --the function to call when a sub menu item is selected
		
		for index, subMenuItem in pairs( subMenuTable ) do --iterate over the key, value pairs and populate the sub menu
			if ( index > 1 ) then --index=1 is the title used in the main menu. Don't need to add it to sub menu
				info = UIDropDownMenu_CreateInfo();
				info.text = subMenuItem;
				info.value = subMenuItem;
				info.func = func;
				info.checked = ( subMenuItem == selectedText ); --is this the option that is selected in the drop down?
				UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
			end

		end
	end
end

function W_DropDownHandler_OnClick()
	local selectedText = this.value; --keep track of what we selected ( the text of it )
	
	UIDropDownMenu_SetText(selectedText, DropDownTest); --set the frame text
end

--
-- Set up and handle the Drop downs in the help menu
--
function W_Spell_DropDownHandler_OnShow()
	local dd = this; --the current menu calling the onshow event
	
	UIDropDownMenu_Initialize( dd, W_Spell_DropDownHandler_Initialize );
	local menuItem, macroName = W_Get_Saved_MenuItem( dd:GetName() ); --retrieve the saved spell for this drop down
	if ( macroName ) then --if this was a macro, then it's name is what we use in the drop down menu
		menuItem = macroName;
	end
	UIDropDownMenu_SetText( menuItem, dd);
	UIDropDownMenu_SetWidth( 150, dd );
end

function W_Spell_DropDownHandler_Initialize()
	local dd = getglobal( UIDROPDOWNMENU_OPEN_MENU );
	local selectedText = UIDropDownMenu_GetText( dd ); --get the option that is selected in the dropdown

	if ( UIDROPDOWNMENU_MENU_LEVEL == 1 ) then -- are we initting main menus
		local index, menuItem;
		local charType = UnitClass("player");
		local tableName = ("W_"..charType.."SpellTable"); --figure out which table to use based on char type
		local t = getglobal(tableName); -- a reference to the table
		local func = function() W_Spell_DropDownHandler_OnClick( dd ) end; --the function to be called when a dropdown item is selected

		for index, menuItem in pairs( t ) do --iterate over the key, values pairs from the spell table for this char
			info = UIDropDownMenu_CreateInfo();

			if ( type(menuItem) == "table" ) then --if this menu item is a table,then add the sub menu arrow
				info.text = menuItem[1]; --the first item in the sub table is the name we want displayed in the mail menu
				info.value = menuItem; --keep track of the subtable that the sub values are in so we can show them when the user selects it
				info.func = nil; --since this is a sub table, there is not function
				info.hasArrow = 1; --it does have an arrow since it is a sub menu
			elseif ( menuItem == W_kMenuRunMacro ) then
				info.text = W_kMenuRunMacro; --the first item in the sub table is the name we want displayed in the mail menu
				info.value = W_kMenuRunMacro; --keep track of the subtable that the sub values are in so we can show them when the user selects it
				info.func = nil; --since this is a sub table, there is not function
				info.hasArrow = 1; --it does have an arrow since it is a sub menu
			else --it was not a table, so add it as a regular menu item
				info.text =  menuItem; --the text to show in the menu
				info.value = menuItem; --the value ( the index ) of the item
				info.func = func; --the function to call when the item is selected
				info.checked = (menuItem == selectedText) --if this is the item that is selected, then check it
			end
			UIDropDownMenu_AddButton( info, 1 );
		end
	elseif ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then -- are we initting sub menus
		local menuItem = UIDROPDOWNMENU_MENU_VALUE; --the value for this is set above in info.value=menuitem, where menuitem is a table or kMacro
		if ( type(menuItem) == "table" ) then --if the value a table? If so, then populate the sub menu
			local subMenuName = menuItem[1]; --the first item in the sub menu table is the name
			local func = function() W_Spell_DropDownHandler_OnClick( dd ) end; --the function to call when a sub menu item is selected
			
			for index, subMenuItem in pairs( menuItem ) do --iterate over the key, value pairs and populate the sub menu
				if ( index > 1 ) then --index=1 is the title used in the main menu. Don't need to add it to sub menu
					info = UIDropDownMenu_CreateInfo();
					info.text = subMenuItem;
					info.value = subMenuItem;
					info.func = func;
					info.checked = ( subMenuItem == selectedText ); --is this the option that is selected in the drop down?
					UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
				end
			end
		elseif ( menuItem == W_kMenuRunMacro ) then
			local func = function() W_Macro_DropDownHandler_OnClick( dd ) end; --the function to call when a sub menu item is selected
			
			local numGlobal, numPerChar = GetNumMacros(); --get the total number of macros total and for this char
			for macroNum = 1,numGlobal do
				local macroName, texture, macrobody, localVar = GetMacroInfo(macroNum)
				local info = UIDropDownMenu_CreateInfo();
				
				info.text = macroName;
				info.value = macroName;

				info.checked = ( macroName == selectedText ); --see which was selected before ( W_DropDownList1,2etc )
				info.func = func;
				UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
			end
		end
	end
end

function W_Spell_DropDownHandler_OnClick( dd )
	local selectedText = this.value; --keep track of what we selected ( the text of it )
	local ddName = dd:GetName();
	
	UIDropDownMenu_SetText(selectedText, dd); --set the frame text to the new selection
	W_SavedSpells[ddName] = selectedText; --store the newly selected spell with this dropdown in the saved spells table
	W_Update_Unit_Buttons(); --update the unit buttons to reflect this new spell
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		HideDropDownMenu(1); --hide the main drop down since we are done selecting something from the sub menu
	end
end


function W_Macro_DropDownHandler_OnClick( dd )
	local macroName = this.value; --keep track of what macro we selected ( the text of it )
	local ddName = dd:GetName();
	
	UIDropDownMenu_SetText(macroName, dd); --set the frame text to the new selection
	W_SavedSpells[ ddName ] = W_kMenuRunMacro; --store the newly selected spell with this dropdown in the saved spells table
	W_gSavedMacros[ ddName ] = macroName; -- keep track of which Macro it is
	W_Update_Unit_Buttons(); --update the unit buttons to reflect this new spell
	if ( UIDROPDOWNMENU_MENU_LEVEL == 2 ) then
		HideDropDownMenu(1); --hide the main drop down since we are done selecting something from the sub menu
	end
end


--
-- Set up and Handle the Health Level drop down
--
function W_Health_DropDownHandler_OnShow()
	local dd = getglobal("W_Health_DropDown");

	UIDropDownMenu_Initialize( this, W_Health_DropDownHandler_Initialize);
	UIDropDownMenu_SetText( W_gHealthLevel, dd);--init to the saved var
	UIDropDownMenu_SetWidth( 50, dd );
end

function W_Health_DropDownHandler_Initialize()
	local dd = getglobal("W_Health_DropDown");
	local selectedText = UIDropDownMenu_GetText( dd ); --get the option that is selected in the dropdown
	local index, menuItem;
	local t = {"Off","10","20","30","40","50","60","70","80","90"}; --table for the percentage health
	local func = function() W_Health_DropDownHandler_OnClick() end; --the function to be called when a dropdown item is selected
	
	for index, menuItem in pairs( t ) do --iterate over the key, values pairs from the table
		info = UIDropDownMenu_CreateInfo();
		info.text =  menuItem; --the text to show in the menu
		info.value = menuItem; --the value ( the index ) of the item
		info.func = func; --the function to call when the item is selected
		info.checked = (menuItem == selectedText) --if this is the item that is selected, then check it
		UIDropDownMenu_AddButton( info, 1 ); --1 is the menu level
	end
end

function W_Health_DropDownHandler_OnClick()
	local dd = getglobal("W_Health_DropDown");
	local selectedText = this.value; --keep track of what we selected ( the text of it )
	
	UIDropDownMenu_SetText(selectedText, dd); --set the frame text
	
	W_gHealthLevel = selectedText; --keep track of the new selection so it will be saved when we log
end